<?php


/**
 * @file highlighter.filter.inc
 * Inclusion of Text_Highlighter libraries and processing of text.
 */

require_once 'Text/Highlighter.php';
require_once 'Text/Highlighter/Renderer.php';
require_once 'Text/Highlighter/Renderer/Html.php';

/**
 * Process a particular text using an array of settings.
 * @param $settings
 *   An array that must contain these keys:
 *     - language: the language to be used.
 *     - numbers: whether the line numbers should be displayed.
 *     - tabsize: the width of an indentation tab, in spaces.
 * @param $text
 *   The text to be processed.
 */
function highlighter_highlight($settings, $text) {
  if ($settings['language'] == 'plain') {
    $out = '<div class="hl-main"><pre>';
    if ($settings['numbers']) {
      $out .= '<ol class="hl-main">';
      $lines = explode("\n", $text);
      $pad = strlen(count($lines));
      foreach ($lines as $i => $line) {
        $out .= sprintf("%0{$pad}d  %s", $i, $line);
      }
      $out .= '</ol>';
    }
    else {
      $out .= "<pre>$text</pre>";
    }
    $out .= '</div>';
    return $out;
  }

  // Load the language class if possible, or abort.
  if (!include_once 'Text/Highlighter/' . strtoupper($settings['language']) . '.php') {
    drupal_set_message(t('Highlighter class for language %lang could not be loaded. Code was not highlighted.', array('%lang' => $settings['language'])), 'error');
    return $text;
  }
  // Initialize the highlighting engine and make sure it is working.
  $hl =& Text_Highlighter::factory($settings['language']);
  if (!is_callable(array($hl, 'setRenderer'))) {
    drupal_set_message(t('The generated class for the %lang language appears to be corrupted.', array('%lang' => $settings['language'])), 'error');
    return $text;
  }

  // Initialize the rendering engine and set the configuration options. 
  $options = array(
    'numbers' => !empty($settings['numbers']) ? HL_NUMBERS_LI : NULL,
    'tabsize' => !empty($settings['tabsize']) ? $settings['tabsize'] : 2,
  );
  $renderer =& new Text_Highlighter_Renderer_HTML($options);
  $hl->setRenderer($renderer);
  
  // Prepare the text and highlight it.
  $text = trim($text, "\n\r "); // whitespace around the code must be gone.
  $html = $hl->highlight($text);
  
  // With line numbering, the code already has linebreaks. Remove newlines.
  if ($settings['numbers']) {
    $html = str_replace("\n", "", $html);
  }
   
  return $html;
}
